#!/usr/local/bin/perl5

#cred    count of reduced pronunciations
#c!red   count of non-reduced pronunciations
#rate    average rate of speech for relevant (reduced or non-reduced)  pronunications
#h#      next word is h#
#nextrep next word is repetition
#thisrep this word is repetition
#fpause  next word is filled pause
#V-initial next word starts with a vowel (either counting or not-counting fpause/repetition, prob counting?)
#C-initial next word starts with a consonant (either counting or not-counting repetition, prob counting?)

%categories=(
	     "i", {
		 "ay" ,"I1",
		 "aa" ,"I2",
		 "ah" ,"I3",
		 "ax" ,"I4",
		 "NULL","I5",
		 "ae" ,"I3",
		 "eh" ,"I3",
		 "aw" ,"I3",
		 "ah ay" ,"I1",
		 "ay ay" ,"I1",
		 "aa ay" ,"I1",
		 "iy" ,"I3",
		 "ih" ,"I3",
		 "ay pv" ,"I1",
		 "ax ay" ,"I1"},
	     "and", {
		 "ae n" ,"AND4",
		 "eh n" ,"AND5",
		 "ae n d" ,"AND1",
		 "ix n" ,"AND7",
		 "en" ,"AND8",
		 "ax n" ,"AND7",
		 "ih n" ,"AND3",
		 "n" ,"AND7",
		 "NULL" ,"AND9",
		 "ah n" ,"AND6",
		 "ae nx" ,"AND4",
		 "eh nx" ,"AND5",
		 "eh" ,"AND5",
		 "ae" ,"AND4",
		 "eh n d" ,"AND2",
		 "eh n n" ,"AND5",
		 "ix" ,"AND7",
		 "ax" ,"AND7",
		 "ih" ,"AND6",
		 "ax nx" ,"AND7",
		 "ax n n" ,"AND7",
		 "ix nx" ,"AND7",
		 "ae n n" ,"AND4",
		 "ix n d" ,"AND8",
		 "ih nx" ,"AND7",
		 "en d" ,"AND8",
		 "ih n n" ,"AND7",
		 "uh n" ,"AND6",
		 "em" ,"AND7",
		 "ah n d" ,"AND3",
		 "nx" ,"AND7",
		 "en n" ,"AND7",
		 "eh n sil" ,"AND5",
		 "eh m" ,"AND5",
		 "ax n d" ,"AND8",
		 "ae m" ,"AND4",
		 "ae eh n d" ,"AND1",
		 "ae d" ,"AND1"},
	     "the", {
		 "dh ax","THE3",
		 "dh iy","THE1",
		 "dh ix","THE4",
		 "ax","THE3",
		 "dh ah","THE2",
		 "n ax","THE3",
		 "NULL","THE5",
		 "dh ih","THE2",
		 "ix","THE4",
		 "ah","THE2",
		 "dh","THE5",
		 "iy","THE1",
		 "d ax","THE3",
		 "n ah","THE2",
		 "ih","THE2",
		 "dh eh","THE2",
		 "d iy","THE1",
		 "d ix","THE4",
		 "n ix","THE4",
		 "dh uh","THE2",
		 "d ih","THE2",
		 "uh","THE2",
		 "th","THE5",
		 "n iy","THE1",
		 "n ih","THE2",
		 "th ax","THE3",
		 "t ax","THE3",
		 "eh","THE2",
		 "d uh","THE2",
		 "d ah","THE2",
		 "t ix","THE4",
		 "dx ax","THE3",
		 "dh ax sil","THE3",
		 "dh ah sil","THE2"},
	     "that", {
		 "dh ae","THAT4",
		 "dh ae t","THAT1",
		 "dh eh","THAT5",
		 "dh eh t","THAT2",
		 "ae","THAT4",
		 "dh ae dx","THAT1",
		 "dh eh dx","THAT2",
		 "dh ih t","THAT3",
		 "dh ih","THAT6",
		 "dh ix t","THAT7",
		 "eh","THAT5",
		 "dh ix","THAT8",
		 "ae t","THAT1",
		 "eh dx","THAT2",
		 "dh ah","THAT6",
		 "ae dx","THAT1",
		 "dh ix dx","THAT7",
		 "dh ae q","THAT1",
		 "n eh","THAT5",
		 "n ae","THAT4",
		 "eh t","THAT2",
		 "dh ax dx","THAT7",
		 "dh ah t","THAT3",
		 "dh ah dx","THAT3",
		 "ax","THAT8",
		 "ix","THAT8",
		 "ih","THAT6",
		 "dh ax","THAT8",
		 "dh","THAT8",
		 "d ae","THAT4",
		 "n ix","THAT8",
		 "n eh t","THAT2",
		 "dh ih dx","THAT3",
		 "dh ae d","THAT1",
		 "d eh","THAT5",
		 "ax dx","THAT7",
		 "ah","THAT6"},
	     "a", {
		 "ax","A3",
		 "ix","A3",
		 "ah","A2",
		 "NULL","A4",
		 "ey","A1",
		 "ih","A2",
		 "er","A2",
		 "eh","A2",
		 "uw","A2",
		 "uh","A2",
		 "ao","A2",
		 "iy","A2",
		 "ow","A2"},
	     "you", {
		 "y ix","YOU3",
		 "y uw","YOU1",
		 "y ih","YOU2",
		 "ih","YOU2",
		 "y ux","YOU4",
		 "iy","YOU2",
		 "ux","YOU4",
		 "y","YOU3",
		 "ix","YOU3",
		 "NULL","YOU5",
		 "uw","YOU1",
		 "y y uw","YOU1",
		 "y ax","YOU3",
		 "y iy","YOU2",
		 "jh uw","YOU1",
		 "y ah","YOU2",
		 "y y ih","YOU2",
		 "y uw w","YOU1",
		 "y er","YOU2",
		 "jh ux","YOU4",
		 "y uh","YOU2",
		 "y eh","YOU2",
		 "n ix","YOU3"},
	     "to", {
		 "t ax","TO3",
		 "t uw","TO1",
		 "t ix","TO3",
		 "t uh","TO2",
		 "ax","TO3",
		 "dx ax","TO3",
		 "t","TO3",
		 "t ux","TO3",
		 "ix","TO3",
		 "NULL","TO4",
		 "t ih","TO2",
		 "nx ax","TO3",
		 "dx ix","TO3",
		 "t uw w","TO1",
		 "t ah","TO2",
		 "n ax","TO3",
		 "t iy","TO2",
		 "uh","TO2",
		 "ih","TO2",
		 "d ax","TO3",
		 "t er","TO2",
		 "n ix","TO3",
		 "n ih","TO2",
		 "n","TO3",
		 "dx uw","TO1",
		 "dx ih","TO2",
		 "d uh","TO2",
		 "d ix","TO3",
		 "ux","TO3",
		 "t eh","TO2",
		 "dx er","TO2"},
	     "of", {
		 "ax","OF6",
		 "ax v","OF5",
		 "ah v","OF1",
		 "ah","OF2",
		 "NULL","OF10",
		 "ax f","OF5",
		 "ix","OF8",
		 "v","OF9",
		 "uh","OF4",
		 "ax v v","OF5",
		 "ix v","OF7",
		 "ah v v","OF1",
		 "ix f","OF7",
		 "ih","OF4",
		 "uh v","OF3",
		 "f","OF9",
		 "iy","OF4",
		 "er","OF4",
		 "aa","OF4",
		 "ih v","OF3"},
	     "it", {
		 "ih","IT2",
		 "ih t","IT1",
		 "ih dx","IT1",
		 "NULL","IT7",
		 "ix","IT6",
		 "ax","IT6",
		 "uh","IT4",
		 "ih d","IT1",
		 "ax t","IT5",
		 "uh t","IT3",
		 "ix dx","IT5",
		 "ih q","IT1",
		 "ah","IT4",
		 "uh dx","IT3",
		 "ix t","IT5",
		 "ax dx","IT5",
		 "ih t s","IT1",
		 "ih ih t","IT1",
		 "eh","IT4"},
	     "in", {
		 "ih n","IN1",
		 "ix n","IN3",
		 "en","IN3",
		 "ax n","IN3",
		 "n","IN3",
		 "ih","IN2",
		 "ih nx","IN1",
		 "NULL","IN4",
		 "ix nx","IN3",
		 "eh n","IN2",
		 "ax","IN3",
		 "ah n","IN2",
		 "ih m","IN1",
		 "ih ng","IN1",
		 "ih n sil","IN1",
		 "ih n n","IN1",
		 "ih ih n","IN1",
		 "er n","IN2",
		 "ae n","IN2"});

%filledpause=("uh",1,"um",1);

$valid=0;
@data=();
$break=0;
while(<>) {
    chop;
    if (/^SENTENCE:/) {
	$break=1;
	next;
    }
    if (/^WORD:/) {
	($mark,$word,$can,$trig,$unig,$lmdiff,$trsyl,$trphn,$lexsyl,$lexphn,$enrate,$vrate,$nvrate,$m1rate,$m2rate,$mrate,$avrate,@phones)=split;
#	print STDERR "($word)";
	while(<>) {
	    last if /^PHONE:/;
	}
#	print STDERR ".";
	($mark,$basephone,@rest)=split;
	$nextrep=0;
	if ($valid || $#fix>=0) {
#	    print STDERR ".\n";
	    if ($break) {
		$nextrep=0;
		$followsil=1;
		$fp=0;
		$binitial="N";
		$rinitial="N";
		if ($#fix>=0) {
		    foreach $f (@fix) {
			$data{$f}->[10]=$binitial;
			$data{$f}->[11]=$rinitial;
		    }
		    @fix=();
		}
		$bdinitial=$binitial;
		$rdinitial=$rinitial;
	    } else {
		$nextrep=1 if ($word eq $prevword);
		$followsil=0;
		$followsil=1 if ($trsyl ne $prevtrsyl);
		$fp=0+$filledpause{$word};
		$binitial="C";
		$binitial="V" if ($basephone =~ /^[aeiou]/);	
		$binitial="N" if ($followsil);
		$rinitial="C";
		$rinitial="V" if ($phones[0] =~ /^[aeiou]/);
		$rinitial="N" if ($followsil);
		if ($fp || $nextrep) {
		    $bdinitial="X";
		    $rdinitial="X";
		    push(@fix,$#data+1) if $valid;
		} else {
		    if ($#fix>=0) {
			foreach $f (@fix) {
#			    print STDERR "---$word ($f) $binitial $basephone $followsil\n";
			    $data[$f]->[10]=$binitial;
			    $data[$f]->[11]=$rinitial;
			}
			@fix=();
		    }
		    $bdinitial=$binitial;
		    $rdinitial=$rinitial;
		}
	    }
	    push(@data,[$prevword,$prevpron,$prevcat,$prevtrsyl,$thisrep,$nextrep,$followsil,$fp,$binitial,$rinitial,$bdinitial,$rdinitial]) if $valid;
	    $valid=0;
	}
#	print STDERR "\n";
	if (defined($categories{$word})) {
#	    print STDERR "$word\n";
	    $string=join(" ",@phones);
	    if (defined($categories{$word}->{$string})) {
#		print STDERR "$word +$string+\n";
		$valid=1;
		$prevcat=$categories{$word}->{$string};
		$prevword=$word;
		$prevpron=join("_",@phones);
		$thisrep=$nextrep;
	    }
	}
	$prevtrsyl=$trsyl;
	$break=0;
    }
}

if ($valid || $#fix>=0) {
    $nextrep=0;
    $followsil=1;
    $fp=0;
    $binitial="N";
    $rinitial="N";
    if ($#fix>=0) {
	foreach $f (@fix) {
	    $data{$f}->[10]=$binitial;
	    $data{$f}->[11]=$rinitial;
	}
	@fix=();
    }
    $bdinitial=$binitial;
    $rdinitial=$rinitial;
    push(@data,[$prevword,$prevpron,$prevcat,$prevtrsyl,$thisrep,$nextrep,$followsil,$fp,$binitial,$rinitial,$bdinitial,$rdinitial]) if $valid;
}

foreach $d (@data) {
    print join(" ",@{$d}),"\n";
}

		
		
